from ._scarabee import *
from . import _scarabee

__author__ = _scarabee.__author__
__copyright__ = _scarabee.__copyright__
__license__ = _scarabee.__license__
__maintainer__ = _scarabee.__maintainer__
__email__ = _scarabee.__email__
__version__ = _scarabee.__version__
